"""0MQ REP-REP tranposrt."""

import asyncio
import pickle
from typing import Any, List

import msgpack
import ormsgpack
import zmq
import zmq.asyncio

from ...errors.errors import Error
from ...errors.exception import VLException
from ...utils.log import Logger


class ZMQTransport:
    """
    ZMQ transport for rpc.

    Attributes:
        address: server address including protocol
        context: zmq context
        logger: A Logger instance
        socket: zmq socket
        connectTimeout: connect timeout
        sndTimeout: maximum time before a send operation
        rcvTimeout: maximum time before a receive operation
        lingerPeriod: linger period for socket shutdown
    """

    def __init__(self, address: str, logger: Logger, connectTimeout: int, requestTimeout: int, responseTimeout: int):
        """
        Args:
            address: rpc server address
            logger: initialized logger instance
            connectTimeout: connect timeout
            requestTimeout: maximum time before a send operation
            responseTimeout: maximum time before a receive operation
        """

        self.socket = None
        self.address = address
        self.logger = logger
        self.context = zmq.asyncio.Context.instance()
        self.connectTimeout = connectTimeout
        self.sndTimeout = requestTimeout
        self.rcvTimeout = responseTimeout
        self.lingerPeriod = 0

    def close(self):
        """Close client."""
        if self.socket is not None:
            try:
                self.socket.close()
            except Exception as exc:  # pylint: disable-msg=broad-except
                self.logger.error(exc)

        self.socket = None

    async def call(self, command: str, args: List[Any], requestId: str) -> Any:
        """
        Pack command and args, send over REQ socket. In case of error, socket is getting closed.

        Args:
            command: A str, used to distinguish commands
            args: List of positional arguments
            requestId: A str, request id
        """
        if self.socket is None:
            # Maybe use RECONNECT_IVL,  see http://api.zeromq.org/master:zmq-setsockopt
            self.socket = self.context.socket(zmq.REQ)  # pylint: disable=E1101
            self.socket.setsockopt(zmq.CONNECT_TIMEOUT, self.connectTimeout)  # pylint: disable=E1101
            self.socket.setsockopt(zmq.SNDTIMEO, self.sndTimeout)  # pylint: disable=E1101
            self.socket.setsockopt(zmq.RCVTIMEO, self.rcvTimeout)  # pylint: disable=E1101
            self.socket.setsockopt(zmq.LINGER, self.lingerPeriod)
            self.logger.info(f"Connecting to {self.address}")
            self.socket.connect(self.address)

        payload = requestId, command, args

        try:
            message = ormsgpack.packb(payload)  # pylint: disable-msg=no-member
        except Exception:
            self.logger.exception()
            raise VLException(Error.InternalServerError) from None

        try:
            await self.socket.send(message)
        except asyncio.CancelledError:
            self.close()  # recreate socket on next request
            raise
        except Exception as exc:
            self.logger.error(f'Couldn\'t send message for command "{command}" due to: {exc}')
            self.close()  # recreate socket on next request
            raise VLException(Error.InternalServerError) from None

        try:
            response = await self.socket.recv()
        except asyncio.CancelledError:
            self.close()  # recreate socket on next request
            raise
        except Exception as exc:
            self.logger.error(f'Couldn\'t receive response for command "{command}" due to: {exc}')
            self.close()  # recreate socket on next request
            raise VLException(Error.InternalServerError) from None

        try:
            result, error = msgpack.unpackb(response, use_list=False)
        except Exception as exc:
            self.logger.error(f"Bad response: {exc}")
            raise VLException(Error.InternalServerError) from None

        if error is not None:
            exc = pickle.loads(error)
            raise exc

        return result
