"""Module realizes response class"""
from typing import Any, Callable, Dict, Iterator, Optional, Tuple

from sanic.response import HTTPResponse, JSONResponse


class HeaderFilteringResponseMixin:
    """Mixin for headers filtration"""

    def __init__(self, *args, headersFilter: Optional[Callable[[str, str], bool]] = None, **kwargs):
        super().__init__(*args, **kwargs)
        self.headersFilter = headersFilter

    @property
    def processed_headers(self) -> Iterator[Tuple[bytes, bytes]]:
        """
        Filter headers before processing for sending
        Returns:
            Tuple[Tuple[bytes, bytes], ...]
        """
        if self.headersFilter:
            for header, value in list(self.headers.items()):
                if not self.headersFilter(header, value):
                    del self.headers[header]
        return super().processed_headers


class VLHTTPResponse(HeaderFilteringResponseMixin, HTTPResponse):
    """HTTPResponse with headers filtering"""


class VLJSONResponse(HeaderFilteringResponseMixin, JSONResponse):
    """JSONResponse with headers filtering"""


def json(
    body: Any,
    status: int = 200,
    headers: Optional[Dict[str, str]] = None,
    content_type: str = "application/json",
    dumps: Optional[Callable[..., str]] = None,
    headersFilter: Optional[Callable[[str, str], bool]] = None,
    **kwargs: Any,
) -> VLJSONResponse:
    """
    Returns response object with body in json format.

    :param body: Response data to be serialized.
    :param status: Response code.
    :param headers: Custom Headers.
    :param headersFilter: Callable headers filter
    :param kwargs: Remaining arguments that are passed to the json encoder.
    """

    return VLJSONResponse(
        body,
        status=status,
        headers=headers,
        content_type=content_type,
        dumps=dumps,
        headersFilter=headersFilter,
        **kwargs,
    )
